@@ -1,5 +1,5 @@
 /* xdaliclock - a melting digital clock
- * Copyright (c) 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998
+ * Copyright (c) 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 2002, 2003
  *  Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -15,7 +15,12 @@
 # include "config.h"
 #endif
 
+#include <stdlib.h>
+#include <string.h>
 #include <stdio.h>
+#include <sys/time.h>
+#include <time.h>
+
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xos.h>
@@ -30,6 +35,10 @@
 #include "usleep.h"
 #include "vroot.h"
 
+#ifndef _XSCREENSAVER_VROOT_H_
+# error Error!  You have an old version of vroot.h!  Check -I args.
+#endif /* _XSCREENSAVER_VROOT_H_ */
+
 #define clock_usleep screenhack_usleep
 
 #ifdef HAVE_SHAPE
@@ -195,7 +204,10 @@ static Pixmap colon, slash;
 static int colon_char_width;
 static int time_digits [6];
 static int last_time_digits [6];
-static int button_down = 0; /* #### */
+
+static enum date_state { DTime, DDateIn, DDate, DDateOut, DDateOut2,
+                         DDash, DDash2 }
+  display_date;
 
 static Pixmap memory_pig_zeros [9] [10];
 static Pixmap memory_pig_digits [8] [10];
@@ -632,15 +644,22 @@ fill_time_digits (void)
           (test_hack == '-' ? -1 : test_hack - '0');
       test_hack = 0;
     }
-  else if (! button_down)
+  else if (display_date == DTime ||
+           display_date == DDash ||
+           display_date == DDash2)
     {
+      if (display_date == DDash)
+        display_date = DDash2;
+      else if (display_date == DDash2)
+        display_date = DTime;
+
       if (countdown)
         {
           int delta = countdown - clock;
           if (delta < 0) delta = -delta;
           tm->tm_sec = delta % 60;
           tm->tm_min = (delta / 60) % 60;
-          tm->tm_hour = (delta / (60 * 60)) % 60;
+          tm->tm_hour = (delta / (60 * 60)) % 100;
           twelve_hour_time = 0;
         }
       if (twelve_hour_time && tm->tm_hour > 12) tm->tm_hour -= 12;
@@ -664,6 +683,15 @@ fill_time_digits (void)
       y0 = (y0 - (y0 % 10)) / 10;
       y1 = tm->tm_year % 10;
 
+      if (display_date == DDateIn)
+        display_date = DDate;
+      else if (display_date == DDateOut)
+        display_date = DDateOut2;
+      else if (display_date == DDateOut2)
+        display_date = DDash;
+      else if (display_date == DDash)
+        display_date = DDash2;
+
       switch (date_format)
         {
         case MMDDYY:
@@ -824,7 +852,7 @@ initialize_digits (Screen *screen, Visual *visual)
 
 
 static void
-initialize_window (Screen *screen, Window window)
+initialize_window (Screen *screen, Window window, Bool external_p)
 {
   Display *dpy = DisplayOfScreen (screen);
   XWindowAttributes xgwa;
@@ -854,17 +882,17 @@ initialize_window (Screen *screen, Window window)
   else
     attributes.background_pixel = bg_color.pixel;
   attributes.event_mask = (xgwa.your_event_mask |
-                           KeyPressMask | ButtonPressMask |
-                           ButtonReleaseMask | StructureNotifyMask |
-                           ExposureMask );
+                           KeyPressMask | StructureNotifyMask | ExposureMask );
   attributes.do_not_propagate_mask = 0;
 
-  if (window != xgwa.root &&
-      window != RootWindowOfScreen (screen))
-    /* On SGIs, you get BadAccess if you try to frob the root window,
-       or a window owned by another client.  Other servers allow this,
-       however... */
-    XChangeWindowAttributes (dpy, window, attribute_mask, &attributes);
+  /* Select ButtonPress and ButtonRelease events on the window only if no other
+     app has already selected them (only one app can select ButtonPress at a
+     time: BadAccess results.)
+   */
+  if (! (xgwa.all_event_masks & (ButtonPressMask | ButtonReleaseMask)))
+    attributes.event_mask |= (ButtonPressMask | ButtonReleaseMask);
+
+  XChangeWindowAttributes (dpy, window, attribute_mask, &attributes);
 
   /* Center stuff in window correctly when first created */
   {
@@ -883,7 +911,8 @@ initialize_window (Screen *screen, Window window)
       }
   }
 
-  XStoreName (dpy, window, window_title);
+  if (! external_p)
+    XStoreName (dpy, window, window_title);
 
   pixmap = XCreatePixmap (dpy, window,
                           x_offsets [ndigits-1] + character_width + 1,
@@ -938,11 +967,10 @@ initialize_window (Screen *screen, Window window)
                   0, 0, x_offsets [ndigits-1] + character_width + 1,
                   character_height + 1);
 
-            /* XClearWindow() depends on us being able to set the background
-               color of the window, which we can't necessarily do if it is the
-               root window.  So fill instead. */
-            XFillRectangle (dpy, window, window_erase_gc,
-                            0, 0, 32767, 32767);
+  /* XClearWindow() depends on us being able to set the background
+     color of the window, which we can't necessarily do if it is the
+     root window.  So fill instead. */
+  XFillRectangle (dpy, window, window_erase_gc, 0, 0, 32767, 32767);
 }
 
 
@@ -1050,7 +1078,7 @@ wander (void)
 static void
 draw_colon (Display *dpy, Drawable pixmap, Window window)
 {
-  Pixmap glyph = button_down ? slash : colon;
+  Pixmap glyph = (display_date == DTime ? colon : slash);
   XCopyPlane (dpy, glyph, pixmap, pixmap_draw_gc,
               0, 0, character_width, character_height,
               x_offsets [1] + character_width, 0, 1);
@@ -1058,7 +1086,7 @@ draw_colon (Display *dpy, Drawable pixmap, Window window)
     XCopyPlane (dpy, glyph, pixmap, pixmap_draw_gc,
                 0, 0, character_width, character_height,
                 x_offsets [3] + character_width, 0, 1);
-  XStoreName (dpy, window, button_down ? hacked_version : window_title);
+  XStoreName (dpy, window, (glyph == slash ? hacked_version : window_title));
 }
 
 
@@ -1072,7 +1100,7 @@ draw_colon (Display *dpy, Drawable pixmap, Window window)
  */
 
 
-static int event_loop (Screen *, Window);
+static int event_loop (Screen *, Window, Bool);
 
 void
 initialize_digital (Screen *screen, Visual *visual, Colormap cmap,
@@ -1099,14 +1127,14 @@ initialize_digital (Screen *screen, Visual *visual, Colormap cmap,
 
 
 void
-run_digital (Screen *screen, Window window)
+run_digital (Screen *screen, Window window, Bool external_p)
 {
   Display *dpy = DisplayOfScreen (screen);
   int i;
   int ndigits = (minutes_only ? 4 : 6);
   XWindowAttributes xgwa;
   XGetWindowAttributes (dpy, window, &xgwa);
-  initialize_window (screen, window);
+  initialize_window (screen, window, external_p);
 
   if (be_a_pig > 0)
     fill_pig_cache (dpy, window, current_frames [0]);
@@ -1117,7 +1145,8 @@ run_digital (Screen *screen, Window window)
       set_current_scanlines (current_frames [i], clear_frame);
     }
 
-  event_loop (screen, window); /* wait for initial mapwindow event */
+  /* wait for initial mapwindow event */
+  event_loop (screen, window, external_p);
 
   XFillRectangle (dpy, window, window_erase_gc,
                   0, 0, x_offsets [ndigits-1] + character_width,
@@ -1130,10 +1159,13 @@ run_digital (Screen *screen, Window window)
       long clock;
       static int date = 0, was = 0;
       int different_minute;
+      enum date_state odate = display_date;
 
       clock = fill_time_digits ();
 
       different_minute = (time_digits [3] != last_time_digits [3]);
+      if (odate != display_date)
+        draw_colon (dpy, pixmap, window);
 
       for (n = 0; n < ndigits; n++)
         if (time_digits [n] == last_time_digits [n])
@@ -1336,7 +1368,7 @@ run_digital (Screen *screen, Window window)
                       character_height);
 
 #ifdef DRYRUN
-      event_loop (screen, window);
+      event_loop (screen, window, external_p);
 #else /* !DRYRUN */
     if (minutes_only)
       {
@@ -1349,12 +1381,12 @@ run_digital (Screen *screen, Window window)
         time_t now = time ((time_t *) 0);
         struct tm *tm = localtime (&now);
         long target = now + (60 - tm->tm_sec);
-        while ((now = time ((time_t *) 0)) <= target)
+        while ((now = time ((time_t *) 0)) < target)
           {
             /* if event_loop returns true, we need to go and repaint stuff
                right now, instead of waiting for the minute to elapse.
                */
-            if (event_loop (screen, window))
+            if (event_loop (screen, window, external_p))
               break;
 
             if (now == target-1)        /* the home stretch; sync up */
@@ -1385,7 +1417,7 @@ run_digital (Screen *screen, Window window)
             /* if event_loop returns true, we need to go and repaint stuff
                right now, instead of waiting for the second to elapse.
              */
-            if (event_loop (screen, window))
+            if (event_loop (screen, window, external_p))
               break;
           }
       }
@@ -1431,16 +1463,14 @@ fill_borders (Display *dpy, Window window, GC gc,
 }
 
 static int
-event_loop (Screen *screen, Window window)
+event_loop (Screen *screen, Window window, Bool external_p)
 {
   Display *dpy = DisplayOfScreen (screen);
   static int mapped_p = 0;
-  int wait_for_buttonup = button_down;
   int redraw_p = 0;
   XSync (dpy, False);
-  /* #### */
-  while (XPending (dpy) || !(mapped_p || root_p) ||
-         (wait_for_buttonup && button_down))
+
+  while (XPending (dpy) || !(mapped_p || external_p))
     {
       XEvent event;
       XNextEvent (dpy, &event);
@@ -1485,13 +1515,16 @@ event_loop (Screen *screen, Window window)
           }
           break;
         case ButtonPress:
-          button_down |= (1<<event.xbutton.button);
-          if (! wait_for_buttonup) draw_colon (dpy, pixmap, window);
+          display_date = DDateIn;
           redraw_p = 1;
+          draw_colon (dpy, pixmap, window);
           break;
         case ButtonRelease:
-          button_down &= ~(1<<event.xbutton.button);
-          if (! button_down) draw_colon (dpy, pixmap, window);
+          if (display_date == DDate) /* turn off faster if already up */
+            display_date = DDash;
+          else if (display_date != DTime)
+            display_date = DDateOut;
+
           redraw_p = 1;
           break;
         case ConfigureNotify:
