@@ -1,6 +1,6 @@
 /* xdaliclock - a melting digital clock
- * Copyright (c) 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999
- *  Jamie Zawinski <jwz@jwz.org>
+ * Copyright (c) 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 
+ *               2002, 2003 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -24,6 +24,7 @@
 # include "config.h"
 #endif
 
+#include <stdlib.h>
 #include <stdio.h>
 #include <ctype.h>
 #include <time.h>
@@ -39,12 +40,15 @@
 #include "resources.h"
 #include "visual.h"
 
+#ifndef _XSCREENSAVER_VROOT_H_
+# error Error!  You have an old version of vroot.h!  Check -I args.
+#endif /* _XSCREENSAVER_VROOT_H_ */
+
 char *progname;
 char *progclass;
 char *hacked_version, *hacked_version2;
 XrmDatabase db;
 Bool wander_p;
-Bool root_p;
 Bool do_cycle;
 long countdown;
 
@@ -72,6 +76,7 @@ static XrmOptionDescRec options [] = {
   { "-visual",          ".visualID",            XrmoptionSepArg, 0 },
   { "-install",         ".installColormap",     XrmoptionNoArg, "True" },
   { "-title",           ".title",               XrmoptionSepArg, 0  }, /* R3 */
+  { "-window-id",       ".windowID",            XrmoptionSepArg, 0 },
   { "-xrm",             0,                      XrmoptionResArg, 0 },
 
   /* options from digital.c */
@@ -112,7 +117,9 @@ main (int argc, char **argv)
   Colormap cmap;
   unsigned long fg, bg, bd;
   XWindowAttributes xgwa;
+  Bool root_p;
   Bool full_screen_p;
+  Window on_window = 0;
 
 #if (XtSpecificationRelease >= 4)
   int argc_copy = argc;
@@ -199,11 +206,20 @@ main (int argc, char **argv)
   full_screen_p = get_boolean_resource ("fullScreen", "Boolean");
   do_cycle = get_boolean_resource ("cycle", "Cycle");
 
+  {
+    char *s = get_string_resource ("windowID", "WindowID");
+    if (s && *s)
+      on_window = get_integer_resource ("windowID", "WindowID");
+    if (s) free (s);
+  }
+
   countdown = 0;
   {
     char *s = get_string_resource ("countdown", "Countdown");
     if (s && *s)
       {
+        time_t now = time((time_t *) 0);
+        int h100 = (99*60*60) + (59*60) + 59;
         long l = 0;
         char c = 0;
         char mon[255];
@@ -267,16 +283,43 @@ main (int argc, char **argv)
                      progname);
             exit (1);
           }
+
+        if (countdown > now + h100 ||
+            countdown < now - h100)
+          {
+            char *s = ctime ((time_t *) &countdown);
+            char *r = strchr (s, '\n');
+            if (r) *r = 0;
+            fprintf (stderr,
+                     "%s: countdown too distant: '%s' is more\n"
+                     "            than 100 hours away (can't "
+                     "display it on the clock face.)\n",
+                     progname, s);
+            exit (1);
+          }
       }
     if (s) free(s);
   }
 
-  if (root_p)
+  if (on_window)
+    {
+      XWindowAttributes xgwa;
+      window = (Window) on_window;
+      XGetWindowAttributes (dpy, window, &xgwa);
+      cmap = xgwa.colormap;
+      visual = xgwa.visual;
+      screen = xgwa.screen;
+      do_shape = 0;
+      do_overlay = 0;
+    }
+  else if (root_p)
     {
       window = RootWindowOfScreen (screen);
       XGetWindowAttributes (dpy, window, &xgwa);
       cmap = xgwa.colormap;
       visual = xgwa.visual;
+      do_shape = 0;
+      do_overlay = 0;
     }
   else
     {
@@ -307,7 +350,7 @@ main (int argc, char **argv)
 
   initialize_digital (screen, visual, cmap, &fg, &bg, &bd, &w, &h);
 
-  if (root_p)
+  if (root_p || on_window)
     {
       /* `window' has already been set, above. */
       if (toplevel)
@@ -384,7 +427,7 @@ main (int argc, char **argv)
   free(argv_copy);
 #endif
 
-  run_digital (screen, window);
+  run_digital (screen, window, (root_p || on_window));
   return (0);
 }
 
