@@ -1,5 +1,5 @@
-/* Dali Clock - a melting digital clock for the Palm Pilot.
- * Copyright (c) 1991-1999 Jamie Zawinski <jwz@jwz.org>
+/* Dali Clock - a melting digital clock for PalmOS.
+ * Copyright (c) 1991-2002 Jamie Zawinski <jwz@jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -15,17 +15,19 @@
 # include "config.h"
 #endif
 
+#ifdef HACK_FRAMEBUFFER
+  /* needed to access WinHandle->displayAddrV20 for ROMs earlier than 3.5 */
+# define ALLOW_ACCESS_TO_INTERNALS_OF_WINDOWS
+#endif
+
 #include "daliclock.h"
 
-/* Define this so that we get `hwrDisplayWidth' from <Hardware.h>.
-   (We only use that value (which is 160, BTW) with <3.0 ROMs.) */
-#define NON_PORTABLE
 
 /* Make sure Err*Display* are not no-ops. */
 #define ERROR_CHECK_LEVEL ERROR_CHECK_FULL
 
-#include <Pilot.h>
-#include <Hardware.h>
+#include <PalmOS.h>
+/*#include <Hardware.h>*/
 #include <FeatureMgr.h>		/* for FtrGet() */
 
 #ifndef HAVE_APPLAUNCHWITHCOMMAND
@@ -35,6 +37,7 @@
 
 #define ROM_20 0x02003000
 #define ROM_30 0x03003000
+#define ROM_35 0x03503000
 
 
 #define SECONDS_ONLY_MODE
@@ -57,7 +60,7 @@ struct scanline {
   POS left[MAX_SEGS_PER_LINE], right[MAX_SEGS_PER_LINE];
 };
 
-typedef VoidHand FrameHandle;
+typedef MemHandle FrameHandle;
 struct frame {
   struct scanline scanlines [1]; /* scanlines are contiguous here */
 };
@@ -76,7 +79,8 @@ struct daliclock_prefs {
 struct state {
   Boolean inverted_p;
   Boolean twelve_hour_time_p;
-  int show_date_p;   /* 0 = no, 1 = do, 2 = stop soon */
+  enum date_state { DTime, DDateIn, DDate, DDateOut, DDateOut2, DDash }
+    display_date;
   char test_hack;
   int ticks;
   long interval;
@@ -97,7 +101,7 @@ struct state {
   signed char current_digits [6];
   signed char target_digits [6];
 
-  SWord win_width, win_height;
+  Int16 win_width, win_height;
 };
 
 
@@ -346,7 +350,7 @@ free_numbers (struct state *state)
 static void
 fill_target_digits (struct state *state)
 {
-  ULong seconds = TimGetSeconds ();  /* since 1/1/1904 */
+  UInt32 seconds = TimGetSeconds ();  /* since 1/1/1904 */
   DateTimeType date;
 
   TimSecondsToDateTime (seconds, &date);
@@ -362,10 +366,14 @@ fill_target_digits (struct state *state)
                                     : state->test_hack - '0');
       state->test_hack = 0;
     }
-  else if (state->show_date_p == 0)
+  else if (state->display_date == DTime ||
+           state->display_date == DDash)
     {
       Boolean twelve_hour_time_p = state->twelve_hour_time_p;
 
+      if (state->display_date == DDash)
+        state->display_date = DTime;
+
       if (twelve_hour_time_p && date.hour > 12) date.hour -= 12;
       if (twelve_hour_time_p && date.hour == 0) date.hour = 12;
       state->target_digits [0] = (date.hour - (date.hour % 10)) / 10;
@@ -389,8 +397,12 @@ fill_target_digits (struct state *state)
       y0 = (y0 - (y0 % 10)) / 10;
       y1 = date.year % 10;
 
-      if (state->show_date_p == 2)
-        state->show_date_p = 0;
+      if (state->display_date == DDateIn)
+        state->display_date = DDate;
+      if (state->display_date == DDateOut)
+        state->display_date = DDateOut2;
+      else if (state->display_date == DDateOut2)
+        state->display_date = DDash;
 
       switch (state->date_format)
         {
@@ -438,58 +450,61 @@ static unsigned int frame_buffer_bytes_per_line = 0;
 static void
 cache_frame_buffer (void)
 {
-# ifdef HAVE_SCRDISPLAYMODE
-  static int mono_p = -1;
+  static int initted_p = 0;
 
-  if (mono_p == 0)	   /* meaning we know we're not in 1-bit-mode; bail. */
-    return;
-  else if (mono_p == -1)   /* meaning we haven't checked yet */
-    {
-      DWord rom_version = 0;
-      FtrGet (sysFtrCreator, sysFtrNumROMVersion, &rom_version);
+  if (initted_p) return;
+  initted_p = 1;
 
-      if (rom_version < ROM_30)  /* ROM earlier than 3.0 -- assume. */
-        {
-          mono_p = 1;
-          frame_buffer_bytes_per_line = hwrDisplayWidth/8;
-        }
-      else
-        {
-          DWord w = 0, h = 0, d = 0;
-          Boolean color = false;
-          ScrDisplayMode (scrDisplayModeGet, &w, &h, &d, &color);
+  {
+    UInt32 rom_version = 0;
+    FtrGet (sysFtrCreator, sysFtrNumROMVersion, &rom_version);
 
-          mono_p = 0;
-          if (w > 0 && h > 0 && d == 1 && color == false)
-            {
-              mono_p = 1;
-              frame_buffer_bytes_per_line = (w+7) / 8;
-            }
-        }
-    }
-# else  /* !HAVE_SCRDISPLAYMODE */
+    frame_buffer_bytes_per_line = 0;
+    frame_buffer = 0;
 
-  /* Didn't have a 3.0-level library at compile-time -- assume. */
-  frame_buffer_bytes_per_line = hwrDisplayWidth/8;
+    if (rom_version < ROM_30)  /* ROM earlier than 3.0 -- assume. */
+      {
+#       ifndef hwrDisplayWidth
+#         define hwrDisplayWidth 160
+#       endif
+        frame_buffer_bytes_per_line = hwrDisplayWidth/8;
+      }
+    else
+      {
+        UInt32 w = 0, h = 0, d = 0;
+        Boolean color = false;
+        WinScreenMode (winScreenModeGet, &w, &h, &d, &color);
+        if (w > 0 && h > 0 && d == 1 && color == false)
+          frame_buffer_bytes_per_line = (w+7) / 8;
+      }
 
-# endif /* !HAVE_SCRDISPLAYMODE */
+    if (frame_buffer_bytes_per_line > 0)
+      {
+        WinHandle w = WinGetDisplayWindow();
 
-  /* If we got here, then we believe we're running on a monochrome system,
-     and know how to access the frame buffer directly. */
-
-  frame_buffer = (unsigned char *) WinGetDisplayWindow()->displayAddr;
+        if (!w)
+          ;
+        else if (rom_version < ROM_35)
+          frame_buffer = (unsigned char *) w->displayAddrV20;
+        else
+          { /* functions from "3.5 New Feature Set" */
+            BitmapPtr b = WinGetBitmap (w);
+            frame_buffer = (unsigned char *) (b ? BmpGetBits (b) : 0);
+          }
+      }
+  }
 }
 #endif /* HACK_FRAMEBUFFER */
 
 
 static void
-draw_horizontal_line (SWord x1, SWord x2, SWord y, Boolean black_p)
+draw_horizontal_line (Int16 x1, Int16 x2, Int16 y, Boolean black_p)
 {
   if (x1 == x2)
     return;
   else if (x1 > x2)
     {
-      SWord swap = x1;
+      Int16 swap = x1;
       x1 = x2;
       x2 = swap;
     }
@@ -714,7 +729,8 @@ draw_clock (struct state *state, Boolean colonic_p)
 
 # define COLON() \
   if (colonic_p) \
-    draw_frame (state, state->base_frames [state->show_date_p ? 11 : 10], \
+    draw_frame (state, state->base_frames \
+                         [state->display_date == DTime ? 10 : 11], \
                 x, y); \
   x += state->colon_width
 
@@ -747,7 +763,7 @@ do_settings_form (struct state *state, FormPtr form)
   FieldPtr field;
   ControlPtr dark, light, c12, c24, secs, mdy, dmy, ymd;
 
-  Handle otext, ntext;
+  MemHandle otext, ntext;
   char buf[5];
   char *s;
 
@@ -761,7 +777,7 @@ do_settings_form (struct state *state, FormPtr form)
       return false;
     }
  
-  if (state->ticks > 10)
+  if (state->ticks >= 10)
     {
       buf[0] = ((state->ticks / 10) % 10) + '0';
       buf[1] = (state->ticks % 10) + '0' ;
@@ -773,17 +789,17 @@ do_settings_form (struct state *state, FormPtr form)
       buf[1] = 0;
     }
 
-  ntext = (Handle) MemHandleNew (StrLen(buf)+1);
-  s = (char *) MemHandleLock ((VoidHand) ntext);
+  ntext = (MemHandle) MemHandleNew (StrLen(buf)+1);
+  s = (char *) MemHandleLock ((MemHandle) ntext);
   StrCopy (s, buf);
-  MemHandleUnlock ((VoidHand) ntext);
+  MemHandleUnlock ((MemHandle) ntext);
 
   otext = FldGetTextHandle (field);
   FldSetTextHandle (field, ntext);
   /* FldDrawField (field); */
 
   if (otext) 
-    MemHandleFree ((VoidHand) otext);
+    MemHandleFree ((MemHandle) otext);
 
   /* Write into the checkboxes.
    */
@@ -845,7 +861,7 @@ do_settings_form (struct state *state, FormPtr form)
 
   /* Parse the frames-per-second text.
    */
-  s = (char *) MemHandleLock ((VoidHand) ntext);
+  s = (char *) MemHandleLock ((MemHandle) ntext);
   state->ticks = 0;
   while (*s)
     {
@@ -853,7 +869,7 @@ do_settings_form (struct state *state, FormPtr form)
       if (state->ticks >= 100) break;
       s++;
     }
-  MemHandleUnlock ((VoidHand) ntext);
+  MemHandleUnlock ((MemHandle) ntext);
 
   FrmDeleteForm (form);
 
@@ -885,7 +901,7 @@ do_settings_form (struct state *state, FormPtr form)
 static void
 sync_to_seconds (void)
 {
-  ULong sec1, sec2;
+  UInt32 sec1, sec2;
   sec1 = sec2 = TimGetSeconds ();
   while (sec1 == sec2)
     sec2 = TimGetSeconds ();
@@ -936,13 +952,16 @@ app_handle_event (struct state *state, EventPtr event)
 
     case penDownEvent:
       if (!state->seconds_only_p)
-        state->show_date_p = 1;
+        state->display_date = DDateIn;
       return true;
       break;
 
     case penUpEvent:
-      if (state->show_date_p)
-        state->show_date_p = 2;
+      if (state->display_date == DDate) /* turn off faster if already up */
+        state->display_date = DDash;
+      else if (state->display_date != DTime)
+        state->display_date = DDateOut;
+
       return true;
       break;
 
@@ -979,7 +998,7 @@ initialize (struct state *state)
 {
   SystemPreferencesType sysPrefs;
   struct daliclock_prefs prefs;
-  Word prefs_size;
+  UInt16 prefs_size;
   Boolean got_prefs;
 
   MemSet (state, sizeof(*state), 0);
@@ -1041,7 +1060,7 @@ initialize (struct state *state)
     }
 
   state->redraw_p = true;
-  state->show_date_p = 0;
+  state->display_date = DTime;
   state->test_hack = 0;
   state->interval = SysTicksPerSecond() / state->ticks;
   state->tick = state->ticks;
@@ -1085,7 +1104,7 @@ start_app (struct state *state)
 static void
 stop_app (struct state *state)
 {
-  state->show_date_p = 0;
+  state->display_date = DTime;
   free_numbers (state);
   FrmCloseAllForms ();
 }
@@ -1094,13 +1113,13 @@ stop_app (struct state *state)
 static void
 event_loop (struct state *state)
 {
-  Word error;
+  UInt16 error;
   EventType event;
 
   long delay = 0;
 
   do {
-    ULong start, stop;
+    UInt32 start, stop;
     long elapsed;
 
     EvtGetEvent (&event, delay);
@@ -1124,9 +1143,9 @@ event_loop (struct state *state)
 
 
 static Err
-check_rom_version (DWord required_version, Word launch_flags)
+check_rom_version (UInt32 required_version, UInt16 launch_flags)
 {
-  DWord rom_version = 0;
+  UInt32 rom_version = 0;
   FtrGet (sysFtrCreator, sysFtrNumROMVersion, &rom_version);
   if (rom_version < required_version)
     {
@@ -1141,11 +1160,8 @@ check_rom_version (DWord required_version, Word launch_flags)
              to another safe one.  The sysFileCDefaultApp is considered "safe"
           */
           if (rom_version < ROM_20)
-            {
-              Err err; /* ignored, but referenced by AppLaunchWithCommand! */
-              AppLaunchWithCommand (sysFileCDefaultApp,
-                                    sysAppLaunchCmdNormalLaunch, 0);
-            }
+            AppLaunchWithCommand (sysFileCDefaultApp,
+                                  sysAppLaunchCmdNormalLaunch, 0);
 # endif /* HAVE_APPLAUNCHWITHCOMMAND */
         }
 
@@ -1156,8 +1172,8 @@ check_rom_version (DWord required_version, Word launch_flags)
 
 
 
-DWord 
-PilotMain (Word cmd, Ptr cmdPBP, Word launchFlags)
+UInt32 
+PilotMain (UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
 {
   Err error = check_rom_version (ROM_20, launchFlags);
   if (error)
@@ -1178,3 +1194,7 @@ PilotMain (Word cmd, Ptr cmdPBP, Word launchFlags)
     }
   return 0;
 }
+
+/* pose -load_apps daliclock.prc -run_app 'Dali Clock'
+   remember to set "ReportScreenAccess=0" in ~/.poserrc
+ */
